/**********************************************************************************************
*
*	PT100 Box			(version mit RS232C Netzwerk Treiber )
*	V1.1  7.2.2008
*	(C) 2008 by Lenzen
*
*	Befehle:
*		CM	=  Contiunes Mode  (Fortlaufende Messung der Selektierten Kanaele )
*		ST  =  Stop Continues  ( Stoppt die fortlaufende Messung)
*		A1  =  AutoCalibration (Auto Kalibrierung ein)
*		A0  =                  (Auto Kalibrierung aus)
*		Mx  =  Kanaele auswaehlen
*		Rx  =  Einzelene Kanaele einmal auslesen
*		Nx  =  Netzwerk addresse Setzen
*
**********************************************************************************************/

#include <math.h>
#include <ctype.h>
#include <reg51.h>                /* special function register declarations   */
#include "eeprom.h"

#define uchar	unsigned char
#define word	unsigned int

#define LF '\n'
#define CR '\r'
#define STX 0x03
#define ETX 0x02

#define BOARDCAST 0xff
#define WAITSTX 0
#define WAITADDR 1
#define SELECT 2

#define MYADDR 0				/* EEPROM Addressen */
#define AUTOMODE 1
#define CHANNELS 2

extern	void	InitSerial(void);
extern	uchar	GetChar(void);
extern	void	PutChar(uchar);
extern	uchar	GetCharCount(void);
extern	void	gcvt(float,char*);
extern	void	TempMeasure(uchar);

extern	float	fTempVal[4];	/* die 4 Temperatur werte */
extern	uchar	ucCharCnt;		/* anzahl der Zeichen in der Eingabe Zeile */
extern	char	szInLine[6];	/* eingabe Zeile */
extern	uchar	ucNetWork;		/* Net work status */

void	Strout(char*);			/* Proto Types */
void	SendTemp(uchar);
uchar	ProcessNetWork(void);

sbit	LED = 0x91;				/* Zugriffs LED */
uchar	ucMyAddr;				/* eigene Addresse */

void	main(void)
{
	bit		AutoCalFlag,RunFlag;
	uchar	ucChan,ucChannels,n;
	char	*p;
	float	fa,fb;

	InitEeprom();				/* EEPROM Initialisieren */
	InitSerial();				/* Serielle Schnittstelle auf 9600 Bd initialisieren */
	PT0 = 1;					/* Timer hat hoechste Pioritaet */
	EA = 1;						/* Interrupt freigeben */
	LED = 1;
	if(RdEeprom(AUTOMODE)) AutoCalFlag = 1;	/* Auto Calibration On/Off */
	else				   AutoCalFlag = 0;
	RunFlag = 0;				/* Einzelabfrage Mode */
	fa = 1.0;					/* Kalibrationsfaktoren */
	fb = 0.0;
	ucChannels = RdEeprom(CHANNELS);				
								/* Kaneale einschalten */
	ucNetWork = WAITSTX;		/* Netzwerk wartet auf STX */
	ucCharCnt = 0;				/* an den Anfang der Zeile */
	szInLine[0] = 0;			/* mit 0 vorbesetzen */
	ucMyAddr = RdEeprom(MYADDR); /* eigene Addresse setzen */
	n = 0;

	for(;;) // Forever
	{
		// Calibration Mode eingeschaltet ?
		if (AutoCalFlag)
		{
			TempMeasure(0);								// Werte fuer 0 Grad ermitteln
			TempMeasure(1);								// Werte fuer 100 Grad ermitteln
			fa = 100.0 / (fTempVal[1] - fTempVal[0]);	// werte ausrechnen
			fb = 100.0 - fTempVal[1] * fa;
		}
		// Fortlaufende Messung eingeschaltet ?
		if(RunFlag)
		{
			if (ucChannels & (1 << n))
 			{
				TempMeasure(n);
				if(AutoCalFlag)							// muessen die werte korrigiert werden ?
					fTempVal[n] = fTempVal[n] * fa + fb;
				SendTemp(n);
				if(n < 3) Strout(" ");
			}
			if(n == 3) Strout("\r\n");
			n = (n+1) & 0x03;							// modulo hochzaehlen
		}

		if(ProcessNetWork())							// RS232 Netzwerk bearbeiten
		{
			p = szInLine;
			while(*p == ' ') p++;						// leerzeichen ueberlesen

			switch(*p){									/* Befehl auswerten */

			case 'R':									/* Read channel */
				RunFlag = 0;							/* Continues Mode aus */
				ucChan = *(p+1)-'0';					/* Kanal Nr holen */
				if (ucChan <= 3)
				{
					TempMeasure(ucChan);
					if(AutoCalFlag)						/* muessen die werte korrigiert werden ? */
						fTempVal[ucChan] = fTempVal[ucChan] * fa + fb;
					SendTemp(ucChan);
					Strout("\r\n");
				}
				break;

			case 'S':								/* Stop continues Mode */
				if(*(p+1) == 'T') RunFlag = 0;
				break;

			case 'C':								/* Start continues Mode */
				if(*(p+1) == 'M') RunFlag = 1;
				break;

			case 'M':								/* Channel set for continues Mode */
				p++;
				if(isxdigit(*p))
				{
					if(isdigit(*p)) ucChannels = *p - '0';
					else			ucChannels = *p - 'A' + 10;
					WrEeprom(CHANNELS,ucChannels);
				}
				break;

			case 'A':								/* Calibration mode */
				if(*(p+1) == '0') 
				{
					AutoCalFlag = 0;				/* aus */
					WrEeprom(AUTOMODE,0);
				}
				else
				if(*(p+1) == '1') 
				{
					AutoCalFlag = 1;				/* ein */
					WrEeprom(AUTOMODE,1);
				}
				break;

			case 'N':								/* Set Network Address */
				p++;
				ucChan = 0;
				while(isdigit(*p))
				{
					ucChan = ucChan * 10 + (*p - '0');
					p++;
				}
				WrEeprom(MYADDR,ucChan);			/* Netzwerk addresse speichern */
				break;
			} // switch

		} // if
		
	} // for ever
}
/*******************************************************************************************
*
*	uchar	ProcessNetWork(void)
*	RS232 Betzwerk bedienen
*	Return: 0 -> Noch keine Zeile eingelesen
*			1 - > Zeile steht in szInLine
*
********************************************************************************************/
uchar	ProcessNetWork(void)
{
	uchar	ch;

	if(GetCharCount())		// zeichen da ?
	{
		ch = GetChar();		// zeichen einlesen
		switch(ucNetWork){
		case WAITSTX:		// auf STX Warten
			if(ch == STX) ucNetWork = WAITADDR;
			return(0);
		case WAITADDR:		// auf Addresse Warten
			if(ch == ucMyAddr || ch == BOARDCAST)
			{
				LED = 0;
				ucNetWork = SELECT;
			} else
				ucNetWork = WAITSTX;
			return(0);
		default:
			if(ch == ETX)		// end Transmission ?
			{
				LED = 1;
				ucNetWork = WAITSTX;
			} else
			if(ch == CR)		// Ende des Befehls ?
			{
				ucCharCnt = 0;
				return(1);
			} else
			if(ch != LF)		// alle zeichen sammeln
			{
				if(ucCharCnt < (sizeof(szInLine)-1))
				{
					szInLine[ucCharCnt++] = toupper(ch);
					szInLine[ucCharCnt] = 0;
				}
			}
			return(0);	
		}
	} else
		return(0);
}
/*******************************************************************************************
*
*	void	SendTemp(uchar ucChan)
*	Temperatur ausgeben
*
********************************************************************************************/
void	SendTemp(uchar ucChan)
{
	char	szStr[8];
	gcvt(fTempVal[ucChan],szStr);
	Strout(szStr);
}
/*******************************************************************************************
*
*	void	Strout(char *szStr)
*	String ausgeben
*
********************************************************************************************/
void	Strout(char * szStr)
{
	if(ucNetWork == SELECT)
		while(*szStr)
			PutChar(*szStr++);
}

